package gs

import java.nio.charset.StandardCharsets
import java.nio.file.Files
import java.nio.file.Paths
import sbt._
import sbt.Keys._

object GsPlugin extends AutoPlugin {
  override def trigger = allRequirements

  val autoImport = GsKeys

  import autoImport._

  val ScalaVersionFile: String = ".scala-version"

  override def projectSettings: Seq[Setting[_]] = Seq(
    publishMavenStyle           := true,
    Test / Keys.publishArtifact := false,
    pomIncludeRepository        := Function.const(false),
    scmInfo := Some(
      ScmInfo(
        browseUrl = url(Gs.gitRepo(gsProjectName.value)),
        connection = Gs.gitSsh(gsProjectName.value)
      )
    ),
    homepage             := Some(url(Gs.gitRepo(gsProjectName.value))),
    organizationName     := Gs.OrganizationName,
    organization         := Gs.GroupId,
    organizationHomepage := Some(url("https://garrity.co/")),
    publishTo := {
      val repo = "https://maven.garrity.co/"
      if (Keys.version.value.endsWith("SNAPSHOT"))
        Some("Garrity Software Maven" at repo + "snapshots")
      else
        Some("Garrity Software Maven" at repo + "gs")
    },
    scalacOptions := {
      val prev = Keys.scalacOptions.value
      (prev ++ Gs.CompilerOptions).distinct
    }
  )

  override lazy val buildSettings: Seq[Setting[_]] = Seq(
    writeScalaVersionToFile := {
      val outputFile = ScalaVersionFile
      Files.write(
        Paths.get(outputFile),
        scalaVersion.value.getBytes(StandardCharsets.UTF_8)
      )
    }
  )

}
